/**
 * Copyright (c) 2019-2020 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Alexander Rose <alexander.rose@weirdbyte.de>
 * @author Joan Segura <joan.segura@rcsb.org>
 * @author Yana Rose <yana.rose@rcsb.org>
 * @author Sebastian Bittrich <sebastian.bittrich@rcsb.org>
 */
/**
 * Copyright (C) 2024, Protein Bioinformatics Research Group, HUN-REN RCNS
 *
 * Licensed under CC BY-NC 4.0, see LICENSE file for more info.
 *
 * @author Gabor Tusnady <tusnady.gabor@ttk.hu>
 * @author Csongor Gerdan <gerdan.csongor@ttk.hu>
 */

import { Viewer, ViewerProps } from '@rcsb/rcsb-molstar/build/src/viewer';
import { TmDetColorThemeProvider, updateSiteColors } from '../../TmFv3DApp/tmdet-extension/tmdet-color-theme';
import { MembraneOrientationPreset } from '../../TmFv3DApp/tmdet-extension/behavior';
import { TmDetLabelProvider } from '../../TmFv3DApp/tmdet-extension/labeling';
import { BuiltInTrajectoryFormat } from 'molstar/lib/mol-plugin-state/formats/trajectory';
import { PresetProps } from '@rcsb/rcsb-molstar/build/src/viewer/helpers/preset';
import { Mat4 } from 'molstar/lib/commonjs/mol-math/linear-algebra';
import { TrajectoryHierarchyPresetProvider } from 'molstar/lib/mol-plugin-state/builder/structure/hierarchy-preset';
import { registerRegionDescriptorData } from '../../TmFv3DApp/UniTmpHelper';
import { TmDetRcsbPreset } from '../TmTrajectoryHierarchyPreset';

export class TmViewerStandalone extends Viewer {

    constructor(elementOrId: string | HTMLElement, props: Partial<ViewerProps> = {}, withTmDetBehaviour: boolean = true) {

        super(elementOrId, props);

        if (withTmDetBehaviour) {
            this.initBehaviour();
        }
    }

    public async initBehaviour() {
        let tree = this.plugin.state.behaviors.build();


        this.plugin.builders.structure.representation.registerPreset(MembraneOrientationPreset);
        this.plugin.representation.structure.themes.colorThemeRegistry.add(TmDetColorThemeProvider);
        this.plugin.managers.lociLabels.addProvider(TmDetLabelProvider);

        await this.plugin.runTask(this.plugin.state.behaviors.updateTree(tree, { doNotUpdateCurrent: true, doNotLogTiming: true }));
    }

    loadStructureFromUrl<P, S>(url: string, format: BuiltInTrajectoryFormat, isBinary: boolean, config?: {
        props?: PresetProps & {
            dataLabel?: string;
        };
        matrix?: Mat4;
        reprProvider?: TrajectoryHierarchyPresetProvider<P, S>;
        params?: P;
    }): Promise<any> {
        return super.loadStructureFromUrl(url, format, isBinary, config);
    }

    unselectChains(chains: string[]) {
        console.log(TmDetColorThemeProvider);
    }

    async loadWithUNITMPMembraneRepresentation(params: any) {
        setTimeout(() => { this.plugin.clear(); }, 100); // clear scene after some delay
        updateSiteColors(params.side1);

        setTimeout(() => { (async () => {
            await registerRegionDescriptorData(params.regionDescriptorUrl, params.side1);

            // load structure
            this.loadStructureFromUrl(params.structureUrl, 'mmcif', false, { reprProvider: TmDetRcsbPreset });

        })(); }, 500);
    }

}
